breed [cards card]
breed [players player]
players-own [
  score      ; score of player during one game
  totscore   ; total score of player during one generation
  hand       ; list of cards in hand of player
  pos        ; position of player
  pchallenge ; probability of an agent to challenge
  pcheat     ; probability of an agent to cheat
]
cards-own[
  number     ; number to identify the cards
  value      ; value of the cards
  owner      ; owner of the cards
  dd?        ; double deck of cards
] 
globals[
  nrgen         ; generation of game during simulation
  avgavglg      ; average length of the game over all patches and all iterations
  avgavgoptions ; average number of unique options available during a game over all patches and all iterations
  avglength     ; average length of the game over all patches
  avgnroptions  ; average number of unique options of all patches
  avghd         ; average frequency of half deck rule during last 100 generations
  avgcc         ; average frequency of color change rule during last 100 generations
  avgr          ; average frequency of reverse rule during last 100 generations
  avgs          ; average frequency of skip rule during last 100 generations
  avgd2         ; average frequency of draw 2 rule during last 100 generations
  avgc          ; average frequency of color rule during last 100 generations
  avgd4         ; average frequency of draw 4 rule during last 100 generations
  avgd4a        ; average frequency of draw 4 any time rule during last 100 generations
  avgrandom     ; average frequency of random order rule during last 100 generations
  avgia         ; average frequency of initial number of cards = 7 during last 100 generations
  avgdt         ; average frequency of draw till rule during last 100 generations
  avgop         ; average number of unique options during last 100 generations
  avgl          ; average length of games during last 100 generations
  discard-pile  ; list of cards on discard pile
  draw-pile     ; list of card on draw pile
  listpchallenge ; distribution of probability of challenge in population
  listpcheat    ; distribution of probability of cheating in population
  halfdeck      ; number of patches with half deck rule
  colorrule     ; number of patches with color rule
  draw4         ; number of patches with draw 4 rule
  draw2         ; number of patches with draw 2 rule
  d4any         ; number of patches with draw 4 any time rule
  skip          ; number of patches with skip rule
  drawtill      ; number of patches with draw till rule
  revers        ; number of patches with reverse rule
  colorc        ; number of patches with color change rule
  iniamount     ; number of patches with initial amount is 7 rule
  randomr       ; number of patches with random order of play rule
  listlength    ; distribution of length of games 
  listoptions   ; distribution of number of unique options to throw cards during games
]
patches-own [
  numberturns       ; the number of turns a group has played
  numberoptions     ; the number of unique card options a group has had
  handlist          ; distribution of the number of unique card options
  avglg             ; average length of the game per agent
  avgoptions        ; average number of unique options per agent 
  minhand           ; what is the smallest number of cards hold by any player
  lengthgame        ; number of turns played in the game
  played?           ; group finished playing
  playing?          ; group on patch playing?
  Half-deck         ; half deck rule on or off
  colorchange       ; color change rule on or off
  reverse?          ; reverse rule on or off
  skip?             ; skip rule on or off
  draw2-num         ; draw 2 cards rule on or off
  Color?            ; colors rule on or off
  draw4-num         ; draw 4 cards rule on or off
  d4-anytime?       ; draw 4 cards rule any time on or off
  random?           ; draw random order on or off
  initialamount     ; initial number of cards 7 or 5
  draw-til?         ; draw till rule on or off
  Half-deck-new     ; new status half deck rule
  colorchange-new   ; new status color change rule
  reverse?-new      ; new status reverse rule
  skip?-new         ; new status skip rule
  draw2-num-new     ; new status draw 2 rule
  Color?-new        ; new status color rule
  draw4-num-new     ; new status draw 4 rule
  d4-anytime?-new   ; new status draw 4 any time rule
  random?-new       ; new status random order rule
  initialamount-new ; new status initial amount = 7 or 5 rule
  draw-til?-new     ; new status draw til rule
  maxop             ; maximum number of options per turn
  won?              ; is true if game on patch is over
  cur               ; current player
  dir               ; direction of turns
  tikker            ; number of times a game is played.
  tikkeriter        ; help variable for tikker
  nrcards           ; number of cards of the game
  nrpos             ; number of cards
  start?            ; true when agents are ready for new game
  ccolor            ; current color of cards
  cnumber           ; cyrrent number of card on top of the deck
  live              ; legal cards
  nroptions         ; number of card know to be in allowable possitions
  game              ; number of ganes played 
  minplayer         ; minimum number of players for group
  nra               ; number of agents on patch
  maxnra            ; maximum number of players
  cheat             ; boolean whether there is cheating of not going on in the group
]

to setup
  ca
  set discard-pile 666
  set draw-pile 333
  set listlength [0 0]
  set listoptions [0 0]
  
  ask patches [set nrpos -1]
  create-players N * (max-pxcor + 1) * (max-pycor + 1)
  let bool false
  ask players [
    set pcheat 0
    set pchallenge 0
    while [bool = false]
    [
      setxy random-pxcor random-pycor
      if count players-here <= N [
        set nrpos nrpos + 1
        set pos nrpos
        set bool true
      ]
    ]
    set bool false
  ]

  ask patches [set nrcards 0 set start? true]
  ask cards [set hidden? true]
  create-cards 104 * (max-pxcor + 1) * (max-pycor + 1)

  initcards
  ask patches [
    set dir 1
    set minhand initialamount
    set played? false
    set playing? true
    set handlist []
    let teller 0
    set won? false
    ifelse random-float 1 < 2.0 [set Half-deck true][set Half-deck false]
    ifelse random-float 1 < 0.0 [set colorchange true][set colorchange false]
    ifelse random-float 1 < 0.0 [set reverse? true][set reverse? false]
    ifelse random-float 1 < 0.0 [set skip? true][set skip? false]
    ifelse random-float 1 < 0.0 [set draw2-num true][set draw2-num false]
    ifelse random-float 1 < 0.0 [set Color? true][set Color? false]
    ifelse random-float 1 < 0.0 [set draw4-num true][set draw4-num false]
    ifelse random-float 1 < 0.0 [set d4-anytime? true][set d4-anytime? false]
    ifelse random-float 1 < 0.0 [set random? true][set random? false]
    ifelse random-float 1 < 0.5 [set initialamount 5][set initialamount 7]
    ifelse random-float 1 < 0.0 [set draw-til? true][set draw-til? false]
    while [teller <= 10]
    [
      set handlist lput 0 handlist
      set teller teller + 1
    ]
    set numberturns 0
    set numberoptions 0
    set tikker 1
    set tikkeriter 0
  ]
   ask players [
    set score 0
    set totscore 0
    set hand []
    let teller 0
   ]
  set nrgen 1

  ask patches [
    set nra count players-here
    set avglg 0
    set avgoptions 0
  ]
  set avghd 0
  set avgcc 0
  set avgr 0
  set avgs 0
  set avgd2 0
  set avgc 0
  set avgd4 0
  set avgd4a 0
  set avgrandom 0
  set avgia 0
  set avgdt 0
  set avgop 0
  set avgl 0
  file-open "output9e.txt"
end

to initcards
; define initial distribution of cards in all groups
 let temp 0
 let bool false
 let totalnumber 0
 ask cards [
   while [bool = false]
   [
     setxy random-pxcor random-pycor
     if count cards-here <= 104 [set totalnumber totalnumber + 1 set nrcards nrcards + 1 set bool true]
   ]
   set bool false
 ]
 ask patches [
   set temp 0
   ask cards-here
   [
     set dd? false
     set owner draw-pile
     if temp <= 25 [set color red]
     if temp >= 26 and temp < 52 [set color blue]
     if temp >= 52 and temp < 78 [set color green]
     if temp >= 78 and temp < 104 [set color yellow]
     if member? temp [1 3 5 7 9 11 13 15 17 19 21 23 25 27 29 31 33 35 37 39 41 43 45 47 49 51 53 55 57 59 61 63 65 67 69 71 73 75 77 79 81 83 85 87 89 91 93 95 97 99 101 103][set dd? true]
     if member? temp [0 1 26 27 52 53 78 79][set number 1 set value 1]
     if member? temp [2 3 28 29 54 55 80 81][set number 2 set value 2]
     if member? temp [4 5 30 31 56 57 82 83][set number 3 set value 3]
     if member? temp [6 7 32 33 58 59 84 85][set number 4 set value 4]
     if member? temp [8 9 34 35 60 61 86 87][set number 5 set value 5]
     if member? temp [10 11 36 37 62 63 88 89][set number 6 set value 6]
     if member? temp [12 13 38 39 64 65 90 91][set number 7 set value 7]
     if member? temp [14 15 40 41 66 67 92 93][set number 8 set value 8]
     if member? temp [16 17 42 43 68 69 94 95][set number 9 set value 9]
     if member? temp [18 44 70 96][set number 10 set value 50]
     if member? temp [19 45 71 97][set number 100 set value 50]
     if member? temp [20 21 46 47 72 73 98 99][set number 11 set value 10]
     if member? temp [22 23 48 49 74 75 100 101][set number 12 set value 10]
     if member? temp [24 25 50 51 76 77 102 103][set number 13 set value 10]
     set hidden? true
     set temp temp + 1
    ]
  ]
end

to initcardspatch 
; initialize distribution of cards on a patch
 if (Half-deck) [ask cards-here with [dd?][set owner -1]]
 ask cards-here [if owner != -1 [set owner draw-pile]]
 if colorchange [ask cards-here with [number = 10] [set color black]]
 if colorchange [ask cards-here with [number = 100] [set color black]]
 if reverse? [ask cards-here with [number = 11] [set value 20]]
 if skip? [ask cards-here with [number = 12] [set value 20]]
 if draw2-num [ask cards-here with [number = 13] [set value 20]]
 ask players-here [if color = grey [set color blue] set hand [] set score 0]
 deal
 ask one-of cards-here with[owner = draw-pile][set live self set owner discard-pile set ccolor [color] of self  
 set cnumber [number] of self] 
 set minhand initialamount
 set minplayer 1 
end

to go
; play cards in one patch at the time until game is over in that patch, and until all patches played the game. 
  let patchesplayed 0
  tick
  if count patches with [start?] = (max-pxcor + 1) * (max-pycor + 1) [
    set patchesplayed count patches with [played?]
    ifelse patchesplayed < count patches [
      ask patches [initcardspatch set playing? true set start? false set pcolor black]
      ][stop]
  ]

  ask patches with [playing?] [
    if nra > 0 [set cur (cur + dir) mod nra ] ; hold-on has been player plays twice
    if random? [set cur random nra]
    ask players-here with [pos = (cur + 1) mod nra ] [   
      set cheat false
      if legal = 0 [ifelse (random-float 1 < pcheat) [set cheat true][set cheat false]]
      ifelse (count legal != 0)[play]
      [
        set handlist replace-item 1 handlist (item 1 handlist + 1)
        draw
        if draw-til?
        [
          while [count legal = 0 and (count cards-here with [owner = draw-pile] > 0)]
          [draw]
        ]
        if (count legal != 0)[play]
      ]
      if count cards-here with[owner = draw-pile] = 0 [ask cards-here with[owner = discard-pile][set owner draw-pile]]
    ]
    if won? = true 
    [ ;; add up the scores for the big game
      let lwinner min [score] of players-here
      ask cards-here [if owner != -1 [set owner draw-pile]]
      ask players-here [set score score + sum [value] of hand 
        set totscore totscore + sum [value] of hand
        set hand cards-here with[owner = myself]
      ]
      let nwinner min [score] of players-here
      set tikkeriter tikkeriter + 1
      deal
      set game game + 1
      set playing? false
      set won? false
      set start? true
      set lengthgame lengthgame + tikker       
      set minhand initialamount
      set minplayer 1
      if count players-here = 1 [ask players-here [set score 1]]
    ]
    if count cards-here with [owner = draw-pile] + count cards-here with [owner = discard-pile] = 0
    [
      let legalcount 0
      ask players-here [set legalcount legalcount + count legal]
      if legalcount = 0 [ 
        ask cards-here [if owner != -1 [set owner draw-pile]]
        ask players-here [set hand cards-here with[owner = myself]]
        set tikkeriter tikkeriter + 1
        deal
        set game game + 1
        set playing? false
        set won? false
        set start? true
        set lengthgame lengthgame + tikker       
        set minhand initialamount
        set minplayer 1
      ]
    ]
    if count players-here > 0 [
      if max [score] of players-here > 0.5 [
      
      set tikker 1
      let teller 0
      let prob 0
      while [teller <= 10]
      [
        set numberturns numberturns + item teller handlist
        set numberoptions numberoptions + item teller handlist * teller
        set teller teller + 1
      ]
     ]
    ]
    if tikkeriter = totiterations [
      set pcolor red
      set played? true
      set playing? false       
      stop
    ]
  ]
  ask patches with [playing?] [set tikker tikker + 1] ;keeps track of number of turns
  
  if (count patches with [played?] = (max-pxcor + 1) * (max-pycor + 1)) 
  [ 
    ask patches [
      set avgoptions 0
      if numberturns > 0 [set avgoptions numberoptions / numberturns]
      set avglg (lengthgame / nra )/ totiterations
    ]
       
    set nrgen nrgen + 1
    plotgraphs

    file-print (word nrgen ": halfdeck: " halfdeck " colorrule: " colorrule " draw4: " draw4 " draw2: " draw2 " d4any: " d4any " skip: " skip " drawtill: " drawtill " revers: " revers " colorc: " colorc " iniamount: " iniamount " randomr: " randomr " avgop: " avgnroptions " avgl: " avglength " meanpcheat: " mean [pcheat] of players " meanpchallenge: " mean [pchallenge] of players);

    ask patches [set played? false]
    ; look at neighbors and imitate
    ask patches [
      set handlist []
      let teller 0
      while [teller <= 10]
      [
        set handlist lput 0 handlist
        set teller teller + 1
      ]
      
      if tikkeriter = totiterations [ ; which neighboring group has highest number of options relative to turns.
        let maxoptemp (numberoptions / numberturns)
        set maxop maxoptemp
        let maxlg lengthgame / nra
        let maxneighbor self
        let thispatch self
        ask neighbors [
          if (numberoptions / numberturns) > maxoptemp 
          [
            if (lengthgame / nra) < (tolerance * maxlg) 
            [
              set [maxop] of thispatch (numberoptions / numberturns) 
              set maxoptemp maxop
              set maxneighbor self
            ]
          ]
        ]  
  
        ifelse random-float 1 < pmut [ifelse random-float 1 < 0.5 [set Half-deck-new false][set Half-deck-new true]] [ifelse (random-float 1 < (1 - (numberoptions / numberturns) / maxop)) [set Half-deck-new [Half-deck] of maxneighbor][set Half-deck-new [Half-deck] of self]]
        ifelse random-float 1 < pmut [ifelse random-float 1 < 0.5 [set color?-new false][set color?-new true]] [ifelse (random-float 1 < (1 - (numberoptions / numberturns) / maxop)) [set color?-new [color?] of maxneighbor][set color?-new [color?] of self]]
        ifelse random-float 1 < pmut [ifelse random-float 1 < 0.5 [set draw4-num-new false][set draw4-num-new true]] [ifelse (random-float 1 < (1 - (numberoptions / numberturns) / maxop)) [set draw4-num-new [draw4-num] of maxneighbor][set draw4-num-new [draw4-num] of self]]
        ifelse random-float 1 < pmut [ifelse random-float 1 < 0.5 [set draw2-num-new false][set draw2-num-new true]] [ifelse (random-float 1 < (1 - (numberoptions / numberturns) / maxop)) [set draw2-num-new [draw2-num] of maxneighbor][set draw2-num-new [draw2-num] of self]]
        ifelse random-float 1 < pmut [ifelse random-float 1 < 0.5 [set d4-anytime?-new false][set d4-anytime?-new true]] [ifelse (random-float 1 < (1 - (numberoptions / numberturns) / maxop)) [set d4-anytime?-new  [d4-anytime? ] of maxneighbor][set d4-anytime?-new [d4-anytime?] of self]]
        ifelse random-float 1 < pmut [ifelse random-float 1 < 0.5 [set skip?-new false][set skip?-new true]] [ifelse (random-float 1 < (1 - (numberoptions / numberturns) / maxop)) [set skip?-new [skip?] of maxneighbor][set skip?-new [skip?] of self]]
        ifelse random-float 1 < pmut [ifelse random-float 1 < 0.5 [set draw-til?-new false][set draw-til?-new true]] [ifelse (random-float 1 < (1 - (numberoptions / numberturns) / maxop)) [set draw-til?-new [draw-til?] of maxneighbor][set draw-til?-new [draw-til?] of self]]
        ifelse random-float 1 < pmut [ifelse random-float 1 < 0.5 [set reverse?-new false][set reverse?-new true]] [ifelse (random-float 1 < (1 - (numberoptions / numberturns) / maxop)) [set reverse?-new [reverse?] of maxneighbor][set reverse?-new [reverse?] of self]]
        ifelse random-float 1 < pmut [ifelse random-float 1 < 0.5 [set colorchange-new false][set colorchange-new true]] [ifelse (random-float 1 < (1 - (numberoptions / numberturns) / maxop)) [set colorchange-new [colorchange] of maxneighbor][set colorchange-new [colorchange] of self]]
        ifelse random-float 1 < pmut [ifelse random-float 1 < 0.5 [set initialamount-new 5][set initialamount-new 7]] [ifelse (random-float 1 < (1 - (numberoptions / numberturns) / maxop)) [set initialamount-new [initialamount] of maxneighbor][set initialamount-new [initialamount] of self]]
        ifelse random-float 1 < pmut [ifelse random-float 1 < 0.5 [set random?-new false][set random?-new true]][ifelse (random-float 1 < (1 - (numberoptions / numberturns) / maxop)) [set random?-new [random?] of maxneighbor][set random?-new [random?] of self]]
      ]
      if indevol [
        let maxscore max [totscore] of players-here
        let copypcheat [pcheat] of one-of players-here with [totscore = maxscore]
        let copypchallenge [pchallenge] of one-of players-here with [totscore = maxscore]
        ask players-here [
          set pcheat (lambda * pcheat) + (1 - lambda) * copypcheat + random-normal 0 pmut
          set pchallenge (lambda * pchallenge) + (1 - lambda) * copypchallenge + random-normal 0 pmut
          if pcheat < 0 [set pcheat 0]
          if pcheat > 1 [set pcheat 1]
          if pchallenge < 0 [set pchallenge 0]
          if pchallenge > 1 [set pchallenge 1] 
        ]
      ]
     ]
    
     ask patches [if tikkeriter = totiterations [set tikkeriter 0]]
     ask patches [
       set nra count players-here
       set numberturns 0
       set numberoptions 0
       set avgoptions 0
       set lengthgame 0
       set avglg 0
       set played? false
       set Half-deck Half-deck-new
       set color? color?-new
       set draw4-num draw4-num-new
       set draw2-num draw2-num-new
       set d4-anytime? d4-anytime?-new
       set skip? skip?-new
       set draw-til? draw-til?-new
       set reverse? reverse?-new
       set colorchange colorchange-new
       set initialamount initialamount-new
       set random? random?-new
       set nrpos -1
     ]
     let bool false
     ask players [
       while [bool = false]
       [
         if count players-here <= [nra] of patch-here [
          set nrpos nrpos + 1
          set pos nrpos
          set bool true
         ]
       ]
       set bool false
    ]
  ]
  if nrgen > 5000 [stop]
end

to play ;this is from the context of a player
  set live nobody ;fix for hold on
    if cheat [set handlist replace-item 1 handlist (item 1 handlist + 1) show "cheat"]
    ifelse count legal = 1 and not cheat [ set handlist replace-item 1 handlist (item 1 handlist + 1)  
    ][
    set nroptions 0

    ifelse color? [
      let dummy 0
      ifelse draw4-num [if count legal with [number = 100] > 0 [set nroptions nroptions + 1]]
        [if count legal with [number = 100] > 0 [set dummy dummy + 1]]
      ifelse skip? [if count legal with[(cnumber = 12 and number = 12) or (number = 12 and color = ccolor)] > 0 [set nroptions nroptions + 1]]
        [if count legal with[(cnumber = 12 and number = 12) or (number = 12 and color = ccolor)] > 0 [set dummy dummy + 1]]
      ifelse reverse? [if count legal with [(cnumber = 11 and number = 11) or (number = 11 and color = ccolor)] > 0 [set nroptions nroptions + 1]]
        [if count legal with [(cnumber = 11 and number = 11) or (number = 11 and color = ccolor)] > 0 [set dummy dummy + 1]]
      ifelse colorchange [if count legal with [number = 10] > 0 [set nroptions nroptions + 1]]
        [if count legal with [number = 10] > 0 [set dummy dummy + 1]]
      ifelse draw2-num [if count legal with [(cnumber = 13 and number = 13) or (number = 13 and color = ccolor)] > 0 [set nroptions nroptions + 1]] 
        [if count legal with [(cnumber = 13 and number = 13) or (number = 13 and color = ccolor)] > 0 [set dummy dummy + 1]]
    
      if (dummy + count legal with [number < 10]) > 0 [set nroptions nroptions + 1]
    ][
      ifelse draw4-num [
        ifelse count legal with [number = 100] > 0 
        [
          ifelse count legal > count legal with [number = 100] [set nroptions 2][set nroptions 1]
        ][
          set nroptions 1
        ]
      ][
        set nroptions 1      
      ]
    ]
    if not cheat [
    set handlist replace-item nroptions handlist (item nroptions handlist + 1)]
  ]
  
  ifelse count legal = 1 [set live one-of legal]
  [
    ifelse random-float 1 > prandom
    [
      ifelse smart [
      let curlegal legal
      let bool false
      let curcolor ccolor
      let thiscard 0
      let listtellers [1 2 3 4 5 6 7 8 9]
      let randomteller 0
        ; for each legal card see whether they other person can play. If you can avoid the other to play with a normal card do this, otherwise use a special card
      let teller 9
      while [teller > 0]
      [
        set randomteller random length listtellers
        set randomteller item randomteller listtellers
        set listtellers remove randomteller listtellers
        ask legal with [number = randomteller]
        [
          set thiscard self
          set curcolor color
          if count cards-here with[owner = ((cur + dir) mod N) and (color = curcolor or number = randomteller)] = 0 [set live thiscard set teller -1 set bool true stop]
         ]
         set teller teller - 1
       ]
      if not bool [
        ask legal with [number = 10]
        [
          set thiscard self
          set curcolor color
          if count cards-here with [owner = ((cur + dir) mod N) and (color = curcolor)] = 0 [set live thiscard 
          set bool true stop]
        ]
      ]
      set listtellers [11 12 13 100]
      set randomteller random length listtellers
      set randomteller item randomteller listtellers
      set listtellers remove randomteller listtellers 
      if not bool [ask legal with [number = randomteller][set live self set bool true stop]]
      set randomteller random length listtellers
      set randomteller item randomteller listtellers
      set listtellers remove randomteller listtellers 
      if not bool [ask legal with [number = randomteller][set live self set bool true stop]]
      set randomteller random length listtellers
      set randomteller item randomteller listtellers
      set listtellers remove randomteller listtellers 
      if not bool [ask legal with [number = randomteller][set live self set bool true stop]]
      set randomteller random length listtellers
      set randomteller item randomteller listtellers
      set listtellers remove randomteller listtellers 
      if not bool [ask legal with [number = randomteller][set live self set bool true stop]]
      if not bool [set live one-of legal]
    ] [
      set live max-one-of legal [value]
    ]
    ]
    [
      set live one-of legal
    ] ; to avoid loops and because people dont play perfectly.
  ]                                   
  ask live
  [
    set ccolor color 
    if ccolor != black and ccolor != red and ccolor != blue and ccolor != yellow and ccolor != green  [show "BB" show ccolor stop]
    set cnumber number set owner discard-pile 
    if reverse? [if number = 11 [set dir dir * -1]] ;reverse
    if draw2-num [if number = 13 [ask players-here with [pos = 1 + ((cur + dir) mod nra)][draw-n 2] if skip? [set cur (cur + dir) mod nra]] set tikker tikker + 1 if not cheat [set handlist replace-item 1 handlist (item 1 handlist + 1)]];draw 2
    if skip? [if number = 12 [if skip? [set cur ((cur + dir) mod nra)]]];skip
    if colorchange [if number = 10 [
      let setcolor one-of [hand] of one-of players-here with [pos = (1 + cur) mod nra]
      set ccolor [color] of one-of [hand] of one-of players-here with [pos = (1 + cur) mod nra] 
      if ccolor != black and ccolor != red and ccolor != blue and ccolor != yellow and ccolor != green  [show "CC" show ccolor show count players-here with [pos = (1 + cur) mod nra] show count [hand] of players-here with [pos = (1 + cur) mod nra] stop]
      if ccolor = black [
        set ccolor rand-color]
    ;  if random-float 1 < [pchallenge] of one-of players-here with [pos = (1 + cur) mod nra] [challenge] ; chance of challenging draw 4
    ]];wild
    ifelse not Half-deck [
      if draw4-num [if number = 100 [
      let rndnr random-float 1
      if rndnr < pchallenge and cheat [
        ; agent is cheating and is challenged. cheater draws 4 cards
        ask player (cur mod N) [draw-n [4]]
      ]
      if rndnr < pchallenge and not cheat [
        ; agent is not cheating and is challenged. challenger draws 6 cards
        ask player ((cur + dir) mod N) [draw-n [6]]
      ]
      if rndnr >= pchallenge [
       ; next player draws 4 cards
        ask player ((cur + dir) mod N) [draw-n [4]];wild draw 4
      ]
      set cur (cur + dir) mod N
      set ccolor [color] of one-of [hand] of player cur
      set handlist replace-item 2 handlist (item 2 handlist + 1)
      set tikker tikker + 1
      if ccolor = black [
        set ccolor rand-color]
    ]];wild draw four          
    
     ][
        if draw4-num [if number = 100 [
        set ccolor [color] of one-of [hand] of one-of players-here with [pos = (cur + 1) mod nra] 
        if ccolor = black [
        set ccolor rand-color]
    ]];wild
     ]  
     if artificialcheat [if ccolor != black [if random-float 1 < [pchallenge] of one-of players-here with [pos = (1 + cur) mod nra] [challenge]]]                              
   ]
   set hand cards-here with [owner = myself]

   set minhand 100
   ask players-here [ if count hand < minhand [set minhand count hand]]
   if count hand = minhand
   [
     if ((cur != minplayer) and (minhand < [count hand] of player minplayer)) 
     [
       set minplayer cur
     ]
   ]  
   if count hand = 0 [set won? true]
  if count cards-here with[owner = 333 or owner = 666] < 2[set won? true] ; if there are no cards left game is over
end

to deal
  ask players-here[let temp  0
              while [temp < initialamount] [ask one-of cards-here with[owner = draw-pile][set owner myself]set temp temp + 1]
              set hand cards-here with[owner = myself]
              ]
end
 
to-report legal
 let rep 0
 let teller 0
 
  ifelse (color? or (not color? and cheat and artificialcheat)) or (color? and (not artificialcheat))[ 
   ifelse ccolor = black [
     ifelse d4-anytime? or cheat
     [set rep hand with [number <= 100]]
     [
       ifelse count hand with [number < 100] = 0 [set rep hand with[number <= 100]
     ][
       set rep hand with [number < 100]]
     ]  
    ][ifelse d4-anytime? or cheat
   [
     set rep hand with[color = ccolor or number = cnumber or number = 10 or number = 100]
   ][
     ifelse count hand with [color = ccolor] = 0 [
       set rep hand with[color = ccolor or number = cnumber or number = 10 or number = 100]
     ][
       ifelse cheat and artificialcheat [set rep hand with[color = ccolor or number = cnumber or number = 10 or number = 100]]
       [set rep hand with[color = ccolor or number = cnumber or number = 10]]]
     ]]
  ][
   ifelse d4-anytime? or cheat
   [set rep hand with[number <= 100]]
   [
   ifelse count hand with [number < 100] = 0 [
     set rep hand with[number <= 100]
   ][
     set rep hand with[number < 100]]
   ]
 ]
 if count rep = 0 [set rep hand with[number = 100]]
 report rep
end

to challenge
 ;if he cheated the current player draws 2 else the next player does
 ifelse cheat [ask players-here with [pos = cur mod nra] [draw-n 2]][ask players-here with [pos = ((cur + dir) mod nra)] [draw-n 2]]
 end

to-report rand-color
let r 0
 set r random 4
 if r = 0 [report yellow]
 if r = 1 [report red]
 if r = 2 [report blue]
 if r = 3 [report green]
  end

to-report gini
 let L sort-by [?1 < ?2] [score] of players-here
 let i 0
 let numerator 0
 while [i < N]
  [
  set numerator numerator + ((N + 1 - (i + 1)) * item i L)
  set i i + 1
  ]

 report (1 / nra) * ( nra + 1 - 2 * ( numerator / sum [score] of players-here))
 end
 
 to draw 
  if count cards-here with[owner = draw-pile] = 0 [ask cards-here with[owner = discard-pile][set owner draw-pile]]
  if count cards-here with [owner = draw-pile] > 0 [ask one-of cards-here with[owner = draw-pile][set owner myself ]]
  set hand cards-here with[owner = myself]
  end
to draw-n [num]
 let temp 0
 while[temp < num]
  [
  draw
  set temp temp + 1
  ]
 end
  
to plotgraphs
  set avglength 0
  set avgnroptions 0 
  
  set listlength []
  set listoptions []
  ask patches [
    set avglength avglength + avglg
    set avgnroptions avgnroptions + avgoptions
    set listlength fput avglg listlength
    set listoptions fput avgoptions listoptions
  ]
  set avglength avglength / count patches
  set avgnroptions avgnroptions / count patches 
  set-current-plot "average length"
  set-current-plot-pen "avgl"
  plot avglength
   
  set-current-plot "average options"
  set-current-plot-pen "avgo"
  plot avgnroptions

  set-current-plot "histogram options"
  histogram listoptions
  
  set-current-plot "histogram length"
  histogram listlength
   
  let listpatches []
  set listpchallenge []
  set listpcheat []
  ask patches [
    set listpatches fput count players-here listpatches
  ]
  ask players [
    set listpchallenge fput pchallenge listpchallenge
    set listpcheat fput pcheat listpcheat
  ]
  
  set-current-plot "histogram players"
  histogram listpatches
  
  set-current-plot "pchallenge"
  histogram listpchallenge
  
  set-current-plot "pcheat"
  histogram listpcheat
  
  set halfdeck 0
  set colorrule 0
  set draw4 0
  set draw2 0
  set d4any 0
  set skip 0
  set drawtill 0
  set revers 0
  set colorc 0
  set iniamount 0
  set randomr 0
  ask patches [
    if Half-deck [set halfdeck halfdeck + 1]
    if color? [set colorrule colorrule + 1]
    if draw4-num [set draw4 draw4 + 1]
    if draw2-num [set draw2 draw2 + 1]
    if d4-anytime? [set d4any d4any + 1]
    if skip? [set skip skip + 1]
    if draw-til? [set drawtill drawtill + 1]
    if reverse? [set revers revers + 1]
    if colorchange [set colorc colorc + 1]
    if initialamount = 7 [set iniamount iniamount + 1]
    if random? [set randomr randomr + 1]
  ]
  set-current-plot "rules"
  set-current-plot-pen "half-deck"
  plot halfdeck / count patches
  set-current-plot-pen "color"
  plot colorrule / count patches
  set-current-plot-pen "draw4"
  plot draw4 / count patches
  set-current-plot-pen "draw2"
  plot draw2 / count patches
  set-current-plot-pen "d4any"
  plot d4any / count patches
  set-current-plot-pen "skip"
  plot skip / count patches
  set-current-plot-pen "drawtill"
  plot drawtill / count patches
  set-current-plot-pen "reverse"
  plot revers / count patches
  set-current-plot-pen "colorchange"
  plot colorc / count patches
  set-current-plot-pen "initialamount"
  plot iniamount / count patches
  set-current-plot-pen "random"
  plot randomr / count patches
    
  if nrgen > 1400
  [
    set avghd avghd + 0.01 * halfdeck / count patches
    set avgcc avgcc + 0.01 * colorc / count patches
    set avgr avgr + 0.01 * revers / count patches
    set avgs avgs + 0.01 * skip / count patches
    set avgd2 avgd2 + 0.01 * draw2 / count patches
    set avgc avgc + 0.01 * colorrule / count patches
    set avgd4 avgd4 + 0.01 * draw4 / count patches
    set avgd4a avgd4a + 0.01 * d4any / count patches
    set avgrandom avgrandom + 0.01 * randomr / count patches
    set avgia avgia + 0.01 * iniamount / count patches
    set avgdt avgdt + 0.01 * drawtill / count patches
    set avgop avgop + 0.01 * avgnroptions
    set avgl avgl + 0.01 * avglength
  ]
 end
 
@#$#@#$#@
GRAPHICS-WINDOW
236
10
446
241
-1
-1
20.0
1
5
1
1
1
0
1
1
1
0
9
0
9
0
0
1
ticks

CC-WINDOW
5
695
1111
790
Command Center
0

BUTTON
31
10
137
43
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
49
56
112
89
NIL
go\n
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
119
59
182
92
NIL
go\n
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

PLOT
779
14
1102
164
average options
NIL
NIL
0.0
25.0
1.0
1.3
true
false
PENS
"avgo" 1.0 0 -16777216 true

SLIDER
35
229
207
262
pmut
pmut
0
1
0.01
0.01
1
NIL
HORIZONTAL

SLIDER
28
182
200
215
prandom
prandom
0
1
1
0.01
1
NIL
HORIZONTAL

SWITCH
1
128
104
161
smart
smart
0
1
-1000

SLIDER
28
341
200
374
totiterations
totiterations
1
10000
10
1
1
NIL
HORIZONTAL

PLOT
483
10
771
163
average length
NIL
NIL
0.0
25.0
0.0
50.0
true
false
PENS
"avgl" 1.0 0 -16777216 true

SLIDER
27
303
199
336
tolerance
tolerance
1
10
10
0.1
1
NIL
HORIZONTAL

PLOT
24
470
421
681
rules
NIL
NIL
0.0
10.0
0.0
1.0
true
true
PENS
"half-deck" 1.0 0 -16777216 true
"color" 1.0 0 -2674135 true
"draw4" 1.0 0 -13345367 true
"draw2" 1.0 0 -10899396 true
"d4any" 1.0 0 -5825686 true
"skip" 1.0 0 -7500403 true
"drawtill" 1.0 0 -11221820 true
"reverse" 1.0 0 -2064490 true
"colorchange" 1.0 0 -13791810 true
"initialamount" 1.0 0 -13840069 true
"random" 1.0 0 -6459832 true

PLOT
427
501
769
681
histogram length
NIL
NIL
0.0
100.0
0.0
10.0
true
false
PENS
"default" 5.0 1 -16777216 true

PLOT
775
502
1098
680
histogram options
NIL
NIL
1.0
1.5
0.0
10.0
true
false
PENS
"default" 0.05 1 -16777216 true

SLIDER
221
258
393
291
learningrate
learningrate
0
2
0.04
0.01
1
NIL
HORIZONTAL

SLIDER
32
266
204
299
N
N
0
10
4
1
1
NIL
HORIZONTAL

PLOT
482
164
772
296
histogram players
NIL
NIL
2.0
8.0
0.0
10.0
true
false
PENS
"default" 1.0 1 -16777216 true

SLIDER
3
89
107
122
lambda
lambda
0
1
0.9
0.01
1
NIL
HORIZONTAL

PLOT
425
305
766
496
pchallenge
NIL
NIL
0.0
1.0
0.0
1.0
true
false
PENS
"default" 0.01 1 -16777216 true

PLOT
773
304
1097
497
pcheat
NIL
NIL
0.0
1.0
0.0
1.0
true
false
PENS
"default" 0.01 1 -16777216 true

SWITCH
220
352
323
385
indevol
indevol
0
1
-1000

SWITCH
56
403
185
436
artificialcheat
artificialcheat
0
1
-1000

@#$#@#$#@
This simulates the evolution of rules of shedding games based on cultural group selection. A number of groups play shedding games and evaluate the consequences on the average length and the difficulty of the game agents experience. Starting with agents who randomly throwing cards, comprehensive games similar to UNO evolve.

The implementation is performed by Marco A. Janssen, Arizona State University, May 2010. Copyright (C) 2010 M.A. Janssen

This program is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

link
true
0
Line -7500403 true 150 0 150 300

link direction
true
0
Line -7500403 true 150 150 30 225
Line -7500403 true 150 150 270 225

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.0.4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="1000000000"/>
    <metric>avgentropy</metric>
    <metric>avglg</metric>
    <metric>avgfc</metric>
    <metric>avgoptions</metric>
    <metric>avgavgentropy</metric>
    <metric>avgavglg</metric>
    <metric>avgavgfc</metric>
    <metric>avgavgoptions</metric>
    <enumeratedValueSet variable="skip?">
      <value value="false"/>
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="draw-til?">
      <value value="false"/>
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="colorchange">
      <value value="false"/>
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="d4-anytime?">
      <value value="false"/>
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="draw2-num">
      <value value="false"/>
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Half-deck">
      <value value="false"/>
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Color?">
      <value value="false"/>
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="draw4-num">
      <value value="false"/>
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="pmut">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="N">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reverse?">
      <value value="false"/>
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="random?">
      <value value="false"/>
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initialamount">
      <value value="5"/>
      <value value="7"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="totiterations">
      <value value="10000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="smart">
      <value value="true"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiment 2 " repetitions="10" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>avghd</metric>
    <metric>avgcc</metric>
    <metric>avgr</metric>
    <metric>avgs</metric>
    <metric>avgd2</metric>
    <metric>avgc</metric>
    <metric>avgd4</metric>
    <metric>avgd4a</metric>
    <metric>avgrandom</metric>
    <metric>avgia</metric>
    <metric>avgdt</metric>
    <metric>avgop</metric>
    <metric>avgl</metric>
    <enumeratedValueSet variable="prandom">
      <value value="0"/>
      <value value="1"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiment" repetitions="10" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>nrgen = 5000</exitCondition>
    <metric>halfdeck</metric>
    <metric>colorrule</metric>
    <metric>draw4</metric>
    <metric>draw2</metric>
    <metric>d4any</metric>
    <metric>skip</metric>
    <metric>drawtill</metric>
    <metric>revers</metric>
    <metric>colorc</metric>
    <metric>iniamount</metric>
    <metric>randomr</metric>
    <metric>avgop</metric>
    <metric>avgl</metric>
    <metric>mean listoptions</metric>
    <metric>mean listlength</metric>
    <metric>mean [pcheat] of players</metric>
    <metric>mean [pchallenge] of players</metric>
    <enumeratedValueSet variable="smart">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="pmut">
      <value value="0.01"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="N">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="totiterations">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tolerance">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="prandom">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="moving">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="indevol">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="artificialcheat">
      <value value="true"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
